@@ -1,11 +1,12 @@
 """Tests for distutils.command.upload."""
 import os
 import unittest
-import http.client as httpclient
 from test.support import run_unittest
 
+from distutils.command import upload as upload_mod
 from distutils.command.upload import upload
 from distutils.core import Distribution
+from distutils.log import INFO
 
 from distutils.tests.test_config import PYPIRC, PyPIRCCommandTestCase
 
@@ -37,48 +38,45 @@ index-servers =
 [server1]
 username:me
 """
-class Response(object):
-    def __init__(self, status=200, reason='OK'):
-        self.status = status
-        self.reason = reason
 
-class FakeConnection(object):
+class FakeOpen(object):
 
-    def __init__(self):
-        self.requests = []
-        self.headers = []
-        self.body = ''
+    def __init__(self, url):
+        self.url = url
+        if not isinstance(url, str):
+            self.req = url
+        else:
+            self.req = None
+        self.msg = 'OK'
 
-    def __call__(self, netloc):
-        return self
+    def getheader(self, name, default=None):
+        return {
+            'content-type': 'text/plain; charset=utf-8',
+            }.get(name.lower(), default)
 
-    def connect(self):
-        pass
-    endheaders = connect
+    def read(self):
+        return b'xyzzy'
 
-    def putrequest(self, method, url):
-        self.requests.append((method, url))
+    def getcode(self):
+        return 200
 
-    def putheader(self, name, value):
-        self.headers.append((name, value))
-
-    def send(self, body):
-        self.body = body
-
-    def getresponse(self):
-        return Response()
 
 class uploadTestCase(PyPIRCCommandTestCase):
 
     def setUp(self):
         super(uploadTestCase, self).setUp()
-        self.old_class = httpclient.HTTPSConnection
-        self.conn = httpclient.HTTPSConnection = FakeConnection()
+        self.old_open = upload_mod.urlopen
+        upload_mod.urlopen = self._urlopen
+        self.last_open = None
 
     def tearDown(self):
-        httpclient.HTTPSConnection = self.old_class
+        upload_mod.urlopen = self.old_open
         super(uploadTestCase, self).tearDown()
 
+    def _urlopen(self, url):
+        self.last_open = FakeOpen(url)
+        return self.last_open
+
     def test_finalize_options(self):
 
         # new format
@@ -119,17 +117,23 @@ class uploadTestCase(PyPIRCCommandTestCase):
         # lets run it
         pkg_dir, dist = self.create_dist(dist_files=dist_files)
         cmd = upload(dist)
+        cmd.show_response = 1
         cmd.ensure_finalized()
         cmd.run()
 
         # what did we send ?
-        headers = dict(self.conn.headers)
+        headers = dict(self.last_open.req.headers)
         self.assertEqual(headers['Content-length'], '2087')
         self.assertTrue(headers['Content-type'].startswith('multipart/form-data'))
-        self.assertFalse('\n' in headers['Authorization'])
+        self.assertEqual(self.last_open.req.get_method(), 'POST')
+        self.assertEqual(self.last_open.req.get_full_url(),
+                         'https://pypi.python.org/pypi')
+        self.assertIn(b'xxx', self.last_open.req.data)
+
+        # The PyPI response body was echoed
+        results = self.get_logs(INFO)
+        self.assertIn('xyzzy\n', results[-1])
 
-        self.assertEqual(self.conn.requests, [('POST', '/pypi')])
-        self.assertTrue((b'xxx') in self.conn.body)
 
 def test_suite():
     return unittest.makeSuite(uploadTestCase)
