@@ -23,9 +23,9 @@
 
 #ifndef MS_WINDOWS
 #define UNIX
-# ifdef HAVE_FCNTL_H
+# ifdef __APPLE__
 #  include <fcntl.h>
-# endif /* HAVE_FCNTL_H */
+# endif
 #endif
 
 #ifdef MS_WINDOWS
@@ -78,8 +78,6 @@ my_getpagesize(void)
 #  define MAP_ANONYMOUS MAP_ANON
 #endif
 
-static PyObject *mmap_module_error;
-
 typedef enum
 {
     ACCESS_DEFAULT,
@@ -91,13 +89,14 @@ typedef enum
 typedef struct {
     PyObject_HEAD
     char *      data;
-    Py_ssize_t  size;
-    Py_ssize_t  pos;    /* relative to offset */
+    size_t      size;
+    size_t      pos;    /* relative to offset */
 #ifdef MS_WINDOWS
     PY_LONG_LONG offset;
 #else
     off_t       offset;
 #endif
+    int     exports;
 
 #ifdef MS_WINDOWS
     HANDLE      map_handle;
@@ -131,8 +130,6 @@ mmap_object_dealloc(mmap_object *m_obj)
     if (m_obj->fd >= 0)
         (void) close(m_obj->fd);
     if (m_obj->data!=NULL) {
-        if (m_obj->access != ACCESS_READ && m_obj->access != ACCESS_COPY)
-            msync(m_obj->data, m_obj->size, MS_SYNC);
         munmap(m_obj->data, m_obj->size);
     }
 #endif /* UNIX */
@@ -143,6 +140,11 @@ mmap_object_dealloc(mmap_object *m_obj)
 static PyObject *
 mmap_close_method(mmap_object *self, PyObject *unused)
 {
+    if (self->exports > 0) {
+        PyErr_SetString(PyExc_BufferError, "cannot close "\
+                        "exported pointers exist");
+        return NULL;
+    }
 #ifdef MS_WINDOWS
     /* For each resource we maintain, we need to check
        the value is valid, and if so, free the resource
@@ -204,53 +206,86 @@ mmap_read_byte_method(mmap_object *self,
                       PyObject *unused)
 {
     CHECK_VALID(NULL);
-    if (self->pos >= self->size) {
+    if (self->pos < self->size) {
+        char value = self->data[self->pos];
+        self->pos += 1;
+        return Py_BuildValue("B", (unsigned char)value);
+    } else {
         PyErr_SetString(PyExc_ValueError, "read byte out of range");
         return NULL;
     }
-    return PyString_FromStringAndSize(&self->data[self->pos++], 1);
 }
 
 static PyObject *
 mmap_read_line_method(mmap_object *self,
                       PyObject *unused)
 {
-    Py_ssize_t remaining;
-    char *start, *eol;
+    char *start = self->data+self->pos;
+    char *eof = self->data+self->size;
+    char *eol;
     PyObject *result;
 
     CHECK_VALID(NULL);
 
-    remaining = (self->pos < self->size) ? self->size - self->pos : 0;
-    if (!remaining)
-        return PyString_FromString("");
-    start = self->data + self->pos;
-    eol = memchr(start, '\n', remaining);
+    eol = memchr(start, '\n', self->size - self->pos);
     if (!eol)
-        eol = self->data + self->size;
+        eol = eof;
     else
-        ++eol; /* advance past newline */
-    result = PyString_FromStringAndSize(start, (eol - start));
+        ++eol;                  /* we're interested in the position after the
+                           newline. */
+    result = PyBytes_FromStringAndSize(start, (eol - start));
     self->pos += (eol - start);
     return result;
 }
 
+/* Basically the "n" format code with the ability to turn None into -1. */
+static int
+mmap_convert_ssize_t(PyObject *obj, void *result) {
+    Py_ssize_t limit;
+    if (obj == Py_None) {
+        limit = -1;
+    }
+    else if (PyNumber_Check(obj)) {
+        limit = PyNumber_AsSsize_t(obj, PyExc_OverflowError);
+        if (limit == -1 && PyErr_Occurred())
+            return 0;
+    }
+    else {
+        PyErr_Format(PyExc_TypeError,
+                     "integer argument expected, got '%.200s'",
+                     Py_TYPE(obj)->tp_name);
+        return 0;
+    }
+    *((Py_ssize_t *)result) = limit;
+    return 1;
+}
+
 static PyObject *
 mmap_read_method(mmap_object *self,
                  PyObject *args)
 {
-    Py_ssize_t num_bytes, remaining;
+    Py_ssize_t num_bytes = -1, n;
     PyObject *result;
 
     CHECK_VALID(NULL);
-    if (!PyArg_ParseTuple(args, "n:read", &num_bytes))
-        return NULL;
+    if (!PyArg_ParseTuple(args, "|O&:read", mmap_convert_ssize_t, &num_bytes))
+        return(NULL);
 
     /* silently 'adjust' out-of-range requests */
-    remaining = (self->pos < self->size) ? self->size - self->pos : 0;
-    if (num_bytes < 0 || num_bytes > remaining)
-        num_bytes = remaining;
-    result = PyString_FromStringAndSize(&self->data[self->pos], num_bytes);
+    assert(self->size >= self->pos);
+    n = self->size - self->pos;
+    /* The difference can overflow, only if self->size is greater than
+     * PY_SSIZE_T_MAX.  But then the operation cannot possibly succeed,
+     * because the mapped area and the returned string each need more
+     * than half of the addressable memory.  So we clip the size, and let
+     * the code below raise MemoryError.
+     */
+    if (n < 0)
+        n = PY_SSIZE_T_MAX;
+    if (num_bytes < 0 || num_bytes > n) {
+        num_bytes = n;
+    }
+    result = PyBytes_FromStringAndSize(self->data+self->pos, num_bytes);
     self->pos += num_bytes;
     return result;
 }
@@ -266,7 +301,7 @@ mmap_gfind(mmap_object *self,
     Py_ssize_t len;
 
     CHECK_VALID(NULL);
-    if (!PyArg_ParseTuple(args, reverse ? "s#|nn:rfind" : "s#|nn:find",
+    if (!PyArg_ParseTuple(args, reverse ? "y#|nn:rfind" : "y#|nn:find",
                           &needle, &len, &start, &end)) {
         return NULL;
     } else {
@@ -277,14 +312,14 @@ mmap_gfind(mmap_object *self,
             start += self->size;
         if (start < 0)
             start = 0;
-        else if (start > self->size)
+        else if ((size_t)start > self->size)
             start = self->size;
 
         if (end < 0)
             end += self->size;
         if (end < 0)
             end = 0;
-        else if (end > self->size)
+        else if ((size_t)end > self->size)
             end = self->size;
 
         start_p = self->data + start;
@@ -296,10 +331,10 @@ mmap_gfind(mmap_object *self,
             for (i = 0; i < len && needle[i] == p[i]; ++i)
                 /* nothing */;
             if (i == len) {
-                return PyInt_FromSsize_t(p - self->data);
+                return PyLong_FromSsize_t(p - self->data);
             }
         }
-        return PyInt_FromLong(-1);
+        return PyLong_FromLong(-1);
     }
 }
 
@@ -318,7 +353,7 @@ mmap_rfind_method(mmap_object *self,
 }
 
 static int
-is_writeable(mmap_object *self)
+is_writable(mmap_object *self)
 {
     if (self->access != ACCESS_READ)
         return 1;
@@ -329,6 +364,11 @@ is_writeable(mmap_object *self)
 static int
 is_resizeable(mmap_object *self)
 {
+    if (self->exports > 0) {
+        PyErr_SetString(PyExc_BufferError,
+                        "mmap can't resize with extant buffers exported.");
+        return 0;
+    }
     if ((self->access == ACCESS_WRITE) || (self->access == ACCESS_DEFAULT))
         return 1;
     PyErr_Format(PyExc_TypeError,
@@ -345,18 +385,18 @@ mmap_write_method(mmap_object *self,
     char *data;
 
     CHECK_VALID(NULL);
-    if (!PyArg_ParseTuple(args, "s#:write", &data, &length))
+    if (!PyArg_ParseTuple(args, "y#:write", &data, &length))
         return(NULL);
 
-    if (!is_writeable(self))
+    if (!is_writable(self))
         return NULL;
 
-    if (self->pos > self->size || self->size - self->pos < length) {
+    if ((self->pos + length) > self->size) {
         PyErr_SetString(PyExc_ValueError, "data out of range");
         return NULL;
     }
-    memcpy(&self->data[self->pos], data, length);
-    self->pos += length;
+    memcpy(self->data+self->pos, data, length);
+    self->pos = self->pos+length;
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -368,14 +408,15 @@ mmap_write_byte_method(mmap_object *self,
     char value;
 
     CHECK_VALID(NULL);
-    if (!PyArg_ParseTuple(args, "c:write_byte", &value))
+    if (!PyArg_ParseTuple(args, "b:write_byte", &value))
         return(NULL);
 
-    if (!is_writeable(self))
+    if (!is_writable(self))
         return NULL;
 
     if (self->pos < self->size) {
-        self->data[self->pos++] = value;
+        *(self->data+self->pos) = value;
+        self->pos += 1;
         Py_INCREF(Py_None);
         return Py_None;
     }
@@ -404,11 +445,11 @@ mmap_size_method(mmap_object *self,
                 return PyErr_SetFromWindowsErr(error);
         }
         if (!high && low < LONG_MAX)
-            return PyInt_FromLong((long)low);
+            return PyLong_FromLong((long)low);
         size = (((PY_LONG_LONG)high)<<32) + low;
         return PyLong_FromLongLong(size);
     } else {
-        return PyInt_FromSsize_t(self->size);
+        return PyLong_FromSsize_t(self->size);
     }
 #endif /* MS_WINDOWS */
 
@@ -416,13 +457,13 @@ mmap_size_method(mmap_object *self,
     {
         struct stat buf;
         if (-1 == fstat(self->fd, &buf)) {
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
 #ifdef HAVE_LARGEFILE_SUPPORT
         return PyLong_FromLongLong(buf.st_size);
 #else
-        return PyInt_FromLong(buf.st_size);
+        return PyLong_FromLong(buf.st_size);
 #endif
     }
 #endif /* UNIX */
@@ -446,14 +487,8 @@ mmap_resize_method(mmap_object *self,
     if (!PyArg_ParseTuple(args, "n:resize", &new_size) ||
         !is_resizeable(self)) {
         return NULL;
-    }
-    if (new_size < 0 || PY_SSIZE_T_MAX - new_size < self->offset) {
-        PyErr_SetString(PyExc_ValueError, "new size out of range");
-        return NULL;
-    }
-
-    {
 #ifdef MS_WINDOWS
+    } else {
         DWORD dwErrCode = 0;
         DWORD off_hi, off_lo, newSizeLow, newSizeHigh;
         /* First, unmap the file view */
@@ -503,29 +538,31 @@ mmap_resize_method(mmap_object *self,
 
 #ifdef UNIX
 #ifndef HAVE_MREMAP
+    } else {
         PyErr_SetString(PyExc_SystemError,
                         "mmap: resizing not available--no mremap()");
         return NULL;
 #else
+    } else {
         void *newmap;
 
-        if (self->fd != -1 && ftruncate(self->fd, self->offset + new_size) == -1) {
-            PyErr_SetFromErrno(mmap_module_error);
+        if (ftruncate(self->fd, self->offset + new_size) == -1) {
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
 
 #ifdef MREMAP_MAYMOVE
         newmap = mremap(self->data, self->size, new_size, MREMAP_MAYMOVE);
 #else
-#if defined(__NetBSD__)
-        newmap = mremap(self->data, self->size, self->data, new_size, 0);
-#else
-        newmap = mremap(self->data, self->size, new_size, 0);
-#endif /* __NetBSD__ */
+        #if defined(__NetBSD__)
+            newmap = mremap(self->data, self->size, self->data, new_size, 0);
+        #else
+            newmap = mremap(self->data, self->size, new_size, 0);
+        #endif /* __NetBSD__ */
 #endif
         if (newmap == (void *)-1)
         {
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
         self->data = newmap;
@@ -541,7 +578,7 @@ static PyObject *
 mmap_tell_method(mmap_object *self, PyObject *unused)
 {
     CHECK_VALID(NULL);
-    return PyInt_FromSize_t(self->pos);
+    return PyLong_FromSize_t(self->pos);
 }
 
 static PyObject *
@@ -552,7 +589,7 @@ mmap_flush_method(mmap_object *self, PyObject *args)
     CHECK_VALID(NULL);
     if (!PyArg_ParseTuple(args, "|nn:flush", &offset, &size))
         return NULL;
-    if (size < 0 || offset < 0 || self->size - offset < size) {
+    if ((size_t)(offset + size) > self->size) {
         PyErr_SetString(PyExc_ValueError, "flush values out of range");
         return NULL;
     }
@@ -561,15 +598,15 @@ mmap_flush_method(mmap_object *self, PyObject *args)
         return PyLong_FromLong(0);
 
 #ifdef MS_WINDOWS
-    return PyInt_FromLong((long) FlushViewOfFile(self->data+offset, size));
+    return PyLong_FromLong((long) FlushViewOfFile(self->data+offset, size));
 #elif defined(UNIX)
     /* XXX semantics of return value? */
     /* XXX flags for msync? */
     if (-1 == msync(self->data + offset, size, MS_SYNC)) {
-        PyErr_SetFromErrno(mmap_module_error);
+        PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
-    return PyInt_FromLong(0);
+    return PyLong_FromLong(0);
 #else
     PyErr_SetString(PyExc_ValueError, "flush not supported on this system");
     return NULL;
@@ -585,18 +622,20 @@ mmap_seek_method(mmap_object *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "n|i:seek", &dist, &how))
         return NULL;
     else {
-        Py_ssize_t where;
+        size_t where;
         switch (how) {
         case 0: /* relative to start */
+            if (dist < 0)
+                goto onoutofrange;
             where = dist;
             break;
         case 1: /* relative to current position */
-            if (PY_SSIZE_T_MAX - self->pos < dist)
+            if ((Py_ssize_t)self->pos + dist < 0)
                 goto onoutofrange;
             where = self->pos + dist;
             break;
         case 2: /* relative to end */
-            if (PY_SSIZE_T_MAX - self->size < dist)
+            if ((Py_ssize_t)self->size + dist < 0)
                 goto onoutofrange;
             where = self->size + dist;
             break;
@@ -604,7 +643,7 @@ mmap_seek_method(mmap_object *self, PyObject *args)
             PyErr_SetString(PyExc_ValueError, "unknown seek type");
             return NULL;
         }
-        if (where > self->size || where < 0)
+        if (where > self->size)
             goto onoutofrange;
         self->pos = where;
         Py_INCREF(Py_None);
@@ -619,42 +658,52 @@ mmap_seek_method(mmap_object *self, PyObject *args)
 static PyObject *
 mmap_move_method(mmap_object *self, PyObject *args)
 {
-    Py_ssize_t dest, src, cnt;
+    unsigned long dest, src, cnt;
     CHECK_VALID(NULL);
-    if (!PyArg_ParseTuple(args, "nnn:move", &dest, &src, &cnt) ||
-        !is_writeable(self)) {
+    if (!PyArg_ParseTuple(args, "kkk:move", &dest, &src, &cnt) ||
+        !is_writable(self)) {
         return NULL;
     } else {
         /* bounds check the values */
-        if (dest < 0 || src < 0 || cnt < 0)
-            goto bounds;
-        if (self->size - dest < cnt || self->size - src < cnt)
-            goto bounds;
-
-        memmove(&self->data[dest], &self->data[src], cnt);
-
+        if ((cnt + dest) < cnt || (cnt + src) < cnt ||
+           src > self->size || (src + cnt) > self->size ||
+           dest > self->size || (dest + cnt) > self->size) {
+            PyErr_SetString(PyExc_ValueError,
+                "source, destination, or count out of range");
+            return NULL;
+        }
+        memmove(self->data+dest, self->data+src, cnt);
         Py_INCREF(Py_None);
         return Py_None;
-
-      bounds:
-        PyErr_SetString(PyExc_ValueError,
-                        "source, destination, or count out of range");
-        return NULL;
     }
 }
 
-#ifdef MS_WINDOWS
 static PyObject *
-mmap__sizeof__method(mmap_object *self, void *unused)
+mmap_closed_get(mmap_object *self)
 {
-    Py_ssize_t res;
-
-    res = _PyObject_SIZE(Py_TYPE(self));
-    if (self->tagname)
-        res += strlen(self->tagname) + 1;
-    return PyLong_FromSsize_t(res);
-}
+#ifdef MS_WINDOWS
+    return PyBool_FromLong(self->map_handle == NULL ? 1 : 0);
+#elif defined(UNIX)
+    return PyBool_FromLong(self->data == NULL ? 1 : 0);
 #endif
+}
+
+static PyObject *
+mmap__enter__method(mmap_object *self, PyObject *args)
+{
+    CHECK_VALID(NULL);
+
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+static PyObject *
+mmap__exit__method(PyObject *self, PyObject *args)
+{
+    _Py_IDENTIFIER(close);
+
+    return _PyObject_CallMethodId(self, &PyId_close, NULL);
+}
 
 static struct PyMethodDef mmap_object_methods[] = {
     {"close",           (PyCFunction) mmap_close_method,        METH_NOARGS},
@@ -671,61 +720,34 @@ static struct PyMethodDef mmap_object_methods[] = {
     {"tell",            (PyCFunction) mmap_tell_method,         METH_NOARGS},
     {"write",           (PyCFunction) mmap_write_method,        METH_VARARGS},
     {"write_byte",      (PyCFunction) mmap_write_byte_method,   METH_VARARGS},
-#ifdef MS_WINDOWS
-    {"__sizeof__",      (PyCFunction) mmap__sizeof__method,     METH_NOARGS},
-#endif
+    {"__enter__",       (PyCFunction) mmap__enter__method,      METH_NOARGS},
+    {"__exit__",        (PyCFunction) mmap__exit__method,       METH_VARARGS},
     {NULL,         NULL}       /* sentinel */
 };
 
+static PyGetSetDef mmap_object_getset[] = {
+    {"closed", (getter) mmap_closed_get, NULL, NULL},
+    {NULL}
+};
+
+
 /* Functions for treating an mmap'ed file as a buffer */
 
-static Py_ssize_t
-mmap_buffer_getreadbuf(mmap_object *self, Py_ssize_t index, const void **ptr)
+static int
+mmap_buffer_getbuf(mmap_object *self, Py_buffer *view, int flags)
 {
     CHECK_VALID(-1);
-    if (index != 0) {
-        PyErr_SetString(PyExc_SystemError,
-                        "Accessing non-existent mmap segment");
+    if (PyBuffer_FillInfo(view, (PyObject*)self, self->data, self->size,
+                          (self->access == ACCESS_READ), flags) < 0)
         return -1;
-    }
-    *ptr = self->data;
-    return self->size;
+    self->exports++;
+    return 0;
 }
 
-static Py_ssize_t
-mmap_buffer_getwritebuf(mmap_object *self, Py_ssize_t index, const void **ptr)
+static void
+mmap_buffer_releasebuf(mmap_object *self, Py_buffer *view)
 {
-    CHECK_VALID(-1);
-    if (index != 0) {
-        PyErr_SetString(PyExc_SystemError,
-                        "Accessing non-existent mmap segment");
-        return -1;
-    }
-    if (!is_writeable(self))
-        return -1;
-    *ptr = self->data;
-    return self->size;
-}
-
-static Py_ssize_t
-mmap_buffer_getsegcount(mmap_object *self, Py_ssize_t *lenp)
-{
-    CHECK_VALID(-1);
-    if (lenp)
-        *lenp = self->size;
-    return 1;
-}
-
-static Py_ssize_t
-mmap_buffer_getcharbuffer(mmap_object *self, Py_ssize_t index, const void **ptr)
-{
-    if (index != 0) {
-        PyErr_SetString(PyExc_SystemError,
-                        "accessing non-existent buffer segment");
-        return -1;
-    }
-    *ptr = (const char *)self->data;
-    return self->size;
+    self->exports--;
 }
 
 static Py_ssize_t
@@ -739,29 +761,11 @@ static PyObject *
 mmap_item(mmap_object *self, Py_ssize_t i)
 {
     CHECK_VALID(NULL);
-    if (i < 0 || i >= self->size) {
+    if (i < 0 || (size_t)i >= self->size) {
         PyErr_SetString(PyExc_IndexError, "mmap index out of range");
         return NULL;
     }
-    return PyString_FromStringAndSize(self->data + i, 1);
-}
-
-static PyObject *
-mmap_slice(mmap_object *self, Py_ssize_t ilow, Py_ssize_t ihigh)
-{
-    CHECK_VALID(NULL);
-    if (ilow < 0)
-        ilow = 0;
-    else if (ilow > self->size)
-        ilow = self->size;
-    if (ihigh < 0)
-        ihigh = 0;
-    if (ihigh < ilow)
-        ihigh = ilow;
-    else if (ihigh > self->size)
-        ihigh = self->size;
-
-    return PyString_FromStringAndSize(self->data + ilow, ihigh-ilow);
+    return PyBytes_FromStringAndSize(self->data + i, 1);
 }
 
 static PyObject *
@@ -774,25 +778,25 @@ mmap_subscript(mmap_object *self, PyObject *item)
             return NULL;
         if (i < 0)
             i += self->size;
-        if (i < 0 || i >= self->size) {
+        if (i < 0 || (size_t)i >= self->size) {
             PyErr_SetString(PyExc_IndexError,
                 "mmap index out of range");
             return NULL;
         }
-        return PyString_FromStringAndSize(self->data + i, 1);
+        return PyLong_FromLong(Py_CHARMASK(self->data[i]));
     }
     else if (PySlice_Check(item)) {
         Py_ssize_t start, stop, step, slicelen;
 
-        if (_PySlice_Unpack(item, &start, &stop, &step) < 0) {
+        if (PySlice_GetIndicesEx(item, self->size,
+                         &start, &stop, &step, &slicelen) < 0) {
             return NULL;
         }
-        slicelen = _PySlice_AdjustIndices(self->size, &start, &stop, step);
 
         if (slicelen <= 0)
-            return PyString_FromStringAndSize("", 0);
+            return PyBytes_FromStringAndSize("", 0);
         else if (step == 1)
-            return PyString_FromStringAndSize(self->data + start,
+            return PyBytes_FromStringAndSize(self->data + start,
                                               slicelen);
         else {
             char *result_buf = (char *)PyMem_Malloc(slicelen);
@@ -805,7 +809,7 @@ mmap_subscript(mmap_object *self, PyObject *item)
                  cur += step, i++) {
                 result_buf[i] = self->data[cur];
             }
-            result = PyString_FromStringAndSize(result_buf,
+            result = PyBytes_FromStringAndSize(result_buf,
                                                 slicelen);
             PyMem_Free(result_buf);
             return result;
@@ -836,52 +840,13 @@ mmap_repeat(mmap_object *self, Py_ssize_t n)
     return NULL;
 }
 
-static int
-mmap_ass_slice(mmap_object *self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject *v)
-{
-    const char *buf;
-
-    CHECK_VALID(-1);
-    if (ilow < 0)
-        ilow = 0;
-    else if (ilow > self->size)
-        ilow = self->size;
-    if (ihigh < 0)
-        ihigh = 0;
-    if (ihigh < ilow)
-        ihigh = ilow;
-    else if (ihigh > self->size)
-        ihigh = self->size;
-
-    if (v == NULL) {
-        PyErr_SetString(PyExc_TypeError,
-                        "mmap object doesn't support slice deletion");
-        return -1;
-    }
-    if (! (PyString_Check(v)) ) {
-        PyErr_SetString(PyExc_IndexError,
-                        "mmap slice assignment must be a string");
-        return -1;
-    }
-    if (PyString_Size(v) != (ihigh - ilow)) {
-        PyErr_SetString(PyExc_IndexError,
-                        "mmap slice assignment is wrong size");
-        return -1;
-    }
-    if (!is_writeable(self))
-        return -1;
-    buf = PyString_AsString(v);
-    memcpy(self->data + ilow, buf, ihigh-ilow);
-    return 0;
-}
-
 static int
 mmap_ass_item(mmap_object *self, Py_ssize_t i, PyObject *v)
 {
     const char *buf;
 
     CHECK_VALID(-1);
-    if (i < 0 || i >= self->size) {
+    if (i < 0 || (size_t)i >= self->size) {
         PyErr_SetString(PyExc_IndexError, "mmap index out of range");
         return -1;
     }
@@ -890,14 +855,14 @@ mmap_ass_item(mmap_object *self, Py_ssize_t i, PyObject *v)
                         "mmap object doesn't support item deletion");
         return -1;
     }
-    if (! (PyString_Check(v) && PyString_Size(v)==1) ) {
+    if (! (PyBytes_Check(v) && PyBytes_Size(v)==1) ) {
         PyErr_SetString(PyExc_IndexError,
-                        "mmap assignment must be single-character string");
+                        "mmap assignment must be length-1 bytes()");
         return -1;
     }
-    if (!is_writeable(self))
+    if (!is_writable(self))
         return -1;
-    buf = PyString_AsString(v);
+    buf = PyBytes_AsString(v);
     self->data[i] = buf[0];
     return 0;
 }
@@ -907,82 +872,84 @@ mmap_ass_subscript(mmap_object *self, PyObject *item, PyObject *value)
 {
     CHECK_VALID(-1);
 
+    if (!is_writable(self))
+        return -1;
+
     if (PyIndex_Check(item)) {
         Py_ssize_t i = PyNumber_AsSsize_t(item, PyExc_IndexError);
-        const char *buf;
+        Py_ssize_t v;
 
         if (i == -1 && PyErr_Occurred())
             return -1;
         if (i < 0)
             i += self->size;
-        if (i < 0 || i >= self->size) {
+        if (i < 0 || (size_t)i >= self->size) {
             PyErr_SetString(PyExc_IndexError,
-                "mmap index out of range");
+                            "mmap index out of range");
             return -1;
         }
         if (value == NULL) {
             PyErr_SetString(PyExc_TypeError,
-                "mmap object doesn't support item deletion");
+                            "mmap doesn't support item deletion");
             return -1;
         }
-        if (!PyString_Check(value) || PyString_Size(value) != 1) {
-            PyErr_SetString(PyExc_IndexError,
-              "mmap assignment must be single-character string");
+        if (!PyIndex_Check(value)) {
+            PyErr_SetString(PyExc_TypeError,
+                            "mmap item value must be an int");
             return -1;
         }
-        if (!is_writeable(self))
+        v = PyNumber_AsSsize_t(value, PyExc_TypeError);
+        if (v == -1 && PyErr_Occurred())
             return -1;
-        buf = PyString_AsString(value);
-        self->data[i] = buf[0];
+        if (v < 0 || v > 255) {
+            PyErr_SetString(PyExc_ValueError,
+                            "mmap item value must be "
+                            "in range(0, 256)");
+            return -1;
+        }
+        self->data[i] = (char) v;
         return 0;
     }
     else if (PySlice_Check(item)) {
         Py_ssize_t start, stop, step, slicelen;
+        Py_buffer vbuf;
 
-        if (_PySlice_Unpack(item, &start, &stop, &step) < 0) {
+        if (PySlice_GetIndicesEx(item,
+                                 self->size, &start, &stop,
+                                 &step, &slicelen) < 0) {
             return -1;
         }
-        slicelen = _PySlice_AdjustIndices(self->size, &start, &stop, step);
         if (value == NULL) {
             PyErr_SetString(PyExc_TypeError,
                 "mmap object doesn't support slice deletion");
             return -1;
         }
-        if (!PyString_Check(value)) {
-            PyErr_SetString(PyExc_IndexError,
-                "mmap slice assignment must be a string");
+        if (PyObject_GetBuffer(value, &vbuf, PyBUF_SIMPLE) < 0)
             return -1;
-        }
-        if (PyString_Size(value) != slicelen) {
+        if (vbuf.len != slicelen) {
             PyErr_SetString(PyExc_IndexError,
                 "mmap slice assignment is wrong size");
+            PyBuffer_Release(&vbuf);
             return -1;
         }
-        if (!is_writeable(self))
-            return -1;
 
-        if (slicelen == 0)
-            return 0;
+        if (slicelen == 0) {
+        }
         else if (step == 1) {
-            const char *buf = PyString_AsString(value);
-
-            if (buf == NULL)
-                return -1;
-            memcpy(self->data + start, buf, slicelen);
-            return 0;
+            memcpy(self->data + start, vbuf.buf, slicelen);
         }
         else {
             Py_ssize_t cur, i;
-            const char *buf = PyString_AsString(value);
 
-            if (buf == NULL)
-                return -1;
-            for (cur = start, i = 0; i < slicelen;
-                 cur += step, i++) {
-                self->data[cur] = buf[i];
+            for (cur = start, i = 0;
+                 i < slicelen;
+                 cur += step, i++)
+            {
+                self->data[cur] = ((char *)vbuf.buf)[i];
             }
-            return 0;
         }
+        PyBuffer_Release(&vbuf);
+        return 0;
     }
     else {
         PyErr_SetString(PyExc_TypeError,
@@ -992,13 +959,13 @@ mmap_ass_subscript(mmap_object *self, PyObject *item, PyObject *value)
 }
 
 static PySequenceMethods mmap_as_sequence = {
-    (lenfunc)mmap_length,                      /*sq_length*/
-    (binaryfunc)mmap_concat,                   /*sq_concat*/
-    (ssizeargfunc)mmap_repeat,                 /*sq_repeat*/
-    (ssizeargfunc)mmap_item,                           /*sq_item*/
-    (ssizessizeargfunc)mmap_slice,             /*sq_slice*/
-    (ssizeobjargproc)mmap_ass_item,            /*sq_ass_item*/
-    (ssizessizeobjargproc)mmap_ass_slice,      /*sq_ass_slice*/
+    (lenfunc)mmap_length,            /*sq_length*/
+    (binaryfunc)mmap_concat,         /*sq_concat*/
+    (ssizeargfunc)mmap_repeat,       /*sq_repeat*/
+    (ssizeargfunc)mmap_item,         /*sq_item*/
+    0,                               /*sq_slice*/
+    (ssizeobjargproc)mmap_ass_item,  /*sq_ass_item*/
+    0,                               /*sq_ass_slice*/
 };
 
 static PyMappingMethods mmap_as_mapping = {
@@ -1008,10 +975,8 @@ static PyMappingMethods mmap_as_mapping = {
 };
 
 static PyBufferProcs mmap_as_buffer = {
-    (readbufferproc)mmap_buffer_getreadbuf,
-    (writebufferproc)mmap_buffer_getwritebuf,
-    (segcountproc)mmap_buffer_getsegcount,
-    (charbufferproc)mmap_buffer_getcharbuffer,
+    (getbufferproc)mmap_buffer_getbuf,
+    (releasebufferproc)mmap_buffer_releasebuf,
 };
 
 static PyObject *
@@ -1051,7 +1016,7 @@ static PyTypeObject mmap_object_type = {
     0,                                          /* tp_print */
     0,                                          /* tp_getattr */
     0,                                          /* tp_setattr */
-    0,                                          /* tp_compare */
+    0,                                          /* tp_reserved */
     0,                                          /* tp_repr */
     0,                                          /* tp_as_number */
     &mmap_as_sequence,                          /*tp_as_sequence*/
@@ -1062,7 +1027,7 @@ static PyTypeObject mmap_object_type = {
     PyObject_GenericGetAttr,                    /*tp_getattro*/
     0,                                          /*tp_setattro*/
     &mmap_as_buffer,                            /*tp_as_buffer*/
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GETCHARBUFFER,                   /*tp_flags*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,   /*tp_flags*/
     mmap_doc,                                   /*tp_doc*/
     0,                                          /* tp_traverse */
     0,                                          /* tp_clear */
@@ -1072,19 +1037,45 @@ static PyTypeObject mmap_object_type = {
     0,                                          /* tp_iternext */
     mmap_object_methods,                        /* tp_methods */
     0,                                          /* tp_members */
-    0,                                          /* tp_getset */
+    mmap_object_getset,                         /* tp_getset */
     0,                                          /* tp_base */
     0,                                          /* tp_dict */
     0,                                          /* tp_descr_get */
     0,                                          /* tp_descr_set */
     0,                                          /* tp_dictoffset */
-    0,                                      /* tp_init */
+    0,                                          /* tp_init */
     PyType_GenericAlloc,                        /* tp_alloc */
     new_mmap_object,                            /* tp_new */
-    PyObject_Del,                           /* tp_free */
+    PyObject_Del,                               /* tp_free */
 };
 
 
+/* extract the map size from the given PyObject
+
+   Returns -1 on error, with an appropriate Python exception raised. On
+   success, the map size is returned. */
+static Py_ssize_t
+_GetMapSize(PyObject *o, const char* param)
+{
+    if (o == NULL)
+        return 0;
+    if (PyIndex_Check(o)) {
+        Py_ssize_t i = PyNumber_AsSsize_t(o, PyExc_OverflowError);
+        if (i==-1 && PyErr_Occurred())
+            return -1;
+        if (i < 0) {
+            PyErr_Format(PyExc_OverflowError,
+                            "memory mapped %s must be positive",
+                            param);
+            return -1;
+        }
+        return i;
+    }
+
+    PyErr_SetString(PyExc_TypeError, "map size must be an integral value");
+    return -1;
+}
+
 #ifdef UNIX
 #ifdef HAVE_LARGEFILE_SUPPORT
 #define _Py_PARSE_OFF_T "L"
@@ -1099,24 +1090,23 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
     struct stat st;
 #endif
     mmap_object *m_obj;
+    PyObject *map_size_obj = NULL;
     Py_ssize_t map_size;
     off_t offset = 0;
     int fd, flags = MAP_SHARED, prot = PROT_WRITE | PROT_READ;
     int devzero = -1;
     int access = (int)ACCESS_DEFAULT;
     static char *keywords[] = {"fileno", "length",
-                                     "flags", "prot",
-                                     "access", "offset", NULL};
+                               "flags", "prot",
+                               "access", "offset", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwdict, "in|iii" _Py_PARSE_OFF_T, keywords,
-                                     &fd, &map_size, &flags, &prot,
+    if (!PyArg_ParseTupleAndKeywords(args, kwdict, "iO|iii" _Py_PARSE_OFF_T, keywords,
+                                     &fd, &map_size_obj, &flags, &prot,
                                      &access, &offset))
         return NULL;
-    if (map_size < 0) {
-        PyErr_SetString(PyExc_OverflowError,
-                        "memory mapped length must be positive");
+    map_size = _GetMapSize(map_size_obj, "size");
+    if (map_size < 0)
         return NULL;
-    }
     if (offset < 0) {
         PyErr_SetString(PyExc_OverflowError,
             "memory mapped offset must be positive");
@@ -1188,7 +1178,7 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
                 return NULL;
             }
             map_size = (Py_ssize_t) (st.st_size - offset);
-        } else if (offset > st.st_size || st.st_size - offset < map_size) {
+        } else if (offset + (size_t)map_size > st.st_size) {
             PyErr_SetString(PyExc_ValueError,
                             "mmap length is greater than file size");
             return NULL;
@@ -1198,8 +1188,9 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
     m_obj = (mmap_object *)type->tp_alloc(type, 0);
     if (m_obj == NULL) {return NULL;}
     m_obj->data = NULL;
-    m_obj->size = map_size;
-    m_obj->pos = 0;
+    m_obj->size = (size_t) map_size;
+    m_obj->pos = (size_t) 0;
+    m_obj->exports = 0;
     m_obj->offset = offset;
     if (fd == -1) {
         m_obj->fd = -1;
@@ -1215,7 +1206,7 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
         fd = devzero = open("/dev/zero", O_RDWR);
         if (devzero == -1) {
             Py_DECREF(m_obj);
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
 #endif
@@ -1223,7 +1214,7 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
         m_obj->fd = dup(fd);
         if (m_obj->fd == -1) {
             Py_DECREF(m_obj);
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
     }
@@ -1239,7 +1230,7 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
     if (m_obj->data == (char *)-1) {
         m_obj->data = NULL;
         Py_DECREF(m_obj);
-        PyErr_SetFromErrno(mmap_module_error);
+        PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
     m_obj->access = (access_mode)access;
@@ -1258,6 +1249,7 @@ static PyObject *
 new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
 {
     mmap_object *m_obj;
+    PyObject *map_size_obj = NULL;
     Py_ssize_t map_size;
     PY_LONG_LONG offset = 0, size;
     DWORD off_hi;       /* upper 32 bits of offset */
@@ -1271,11 +1263,11 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
     int access = (access_mode)ACCESS_DEFAULT;
     DWORD flProtect, dwDesiredAccess;
     static char *keywords[] = { "fileno", "length",
-                                      "tagname",
-                                      "access", "offset", NULL };
+                                "tagname",
+                                "access", "offset", NULL };
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwdict, "in|ziL", keywords,
-                                     &fileno, &map_size,
+    if (!PyArg_ParseTupleAndKeywords(args, kwdict, "iO|ziL", keywords,
+                                     &fileno, &map_size_obj,
                                      &tagname, &access, &offset)) {
         return NULL;
     }
@@ -1298,11 +1290,9 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
                             "mmap invalid access parameter.");
     }
 
-    if (map_size < 0) {
-        PyErr_SetString(PyExc_OverflowError,
-                        "memory mapped length must be positive");
+    map_size = _GetMapSize(map_size_obj, "size");
+    if (map_size < 0)
         return NULL;
-    }
     if (offset < 0) {
         PyErr_SetString(PyExc_OverflowError,
             "memory mapped offset must be positive");
@@ -1314,18 +1304,19 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
        XXX: fileno == 0 is a valid fd, but was accepted prior to 2.5.
        XXX: Should this code be added?
        if (fileno == 0)
-        PyErr_Warn(PyExc_DeprecationWarning,
-                   "don't use 0 for anonymous memory");
+        PyErr_WarnEx(PyExc_DeprecationWarning,
+                     "don't use 0 for anonymous memory",
+                     1);
      */
     if (fileno != -1 && fileno != 0) {
         /* Ensure that fileno is within the CRT's valid range */
         if (_PyVerify_fd(fileno) == 0) {
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
         fh = (HANDLE)_get_osfhandle(fileno);
         if (fh==(HANDLE)-1) {
-            PyErr_SetFromErrno(mmap_module_error);
+            PyErr_SetFromErrno(PyExc_OSError);
             return NULL;
         }
         /* Win9x appears to need us seeked to zero */
@@ -1403,6 +1394,7 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
     /* set the initial position */
     m_obj->pos = (size_t) 0;
 
+    m_obj->exports = 0;
     /* set the tag name */
     if (tagname != NULL && *tagname != '\0') {
         m_obj->tagname = PyMem_Malloc(strlen(tagname)+1);
@@ -1453,31 +1445,40 @@ new_mmap_object(PyTypeObject *type, PyObject *args, PyObject *kwdict)
 static void
 setint(PyObject *d, const char *name, long value)
 {
-    PyObject *o = PyInt_FromLong(value);
+    PyObject *o = PyLong_FromLong(value);
     if (o && PyDict_SetItemString(d, name, o) == 0) {
         Py_DECREF(o);
     }
 }
 
+
+static struct PyModuleDef mmapmodule = {
+    PyModuleDef_HEAD_INIT,
+    "mmap",
+    NULL,
+    -1,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
 PyMODINIT_FUNC
-initmmap(void)
+PyInit_mmap(void)
 {
     PyObject *dict, *module;
 
     if (PyType_Ready(&mmap_object_type) < 0)
-        return;
+        return NULL;
 
-    module = Py_InitModule("mmap", NULL);
+    module = PyModule_Create(&mmapmodule);
     if (module == NULL)
-        return;
+        return NULL;
     dict = PyModule_GetDict(module);
     if (!dict)
-        return;
-    mmap_module_error = PyErr_NewException("mmap.error",
-        PyExc_EnvironmentError , NULL);
-    if (mmap_module_error == NULL)
-        return;
-    PyDict_SetItemString(dict, "error", mmap_module_error);
+        return NULL;
+    PyDict_SetItemString(dict, "error", PyExc_OSError);
     PyDict_SetItemString(dict, "mmap", (PyObject*) &mmap_object_type);
 #ifdef PROT_EXEC
     setint(dict, "PROT_EXEC", PROT_EXEC);
@@ -1513,4 +1514,5 @@ initmmap(void)
     setint(dict, "ACCESS_READ", ACCESS_READ);
     setint(dict, "ACCESS_WRITE", ACCESS_WRITE);
     setint(dict, "ACCESS_COPY", ACCESS_COPY);
+    return module;
 }
