@@ -1,6 +1,5 @@
 
 #include "Python.h"
-#include "structseq.h"
 #include <sys/resource.h>
 #include <sys/time.h>
 #include <string.h>
@@ -19,8 +18,6 @@
 
 #define doubletime(TV) ((double)(TV).tv_sec + (TV).tv_usec * 0.000001)
 
-static PyObject *ResourceError;
-
 PyDoc_STRVAR(struct_rusage__doc__,
 "struct_rusage: Result from getrusage.\n\n"
 "This object may be accessed either as a tuple of\n"
@@ -74,7 +71,7 @@ resource_getrusage(PyObject *self, PyObject *args)
                             "invalid who parameter");
             return NULL;
         }
-        PyErr_SetFromErrno(ResourceError);
+        PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
 
@@ -86,20 +83,20 @@ resource_getrusage(PyObject *self, PyObject *args)
                     PyFloat_FromDouble(doubletime(ru.ru_utime)));
     PyStructSequence_SET_ITEM(result, 1,
                     PyFloat_FromDouble(doubletime(ru.ru_stime)));
-    PyStructSequence_SET_ITEM(result, 2, PyInt_FromLong(ru.ru_maxrss));
-    PyStructSequence_SET_ITEM(result, 3, PyInt_FromLong(ru.ru_ixrss));
-    PyStructSequence_SET_ITEM(result, 4, PyInt_FromLong(ru.ru_idrss));
-    PyStructSequence_SET_ITEM(result, 5, PyInt_FromLong(ru.ru_isrss));
-    PyStructSequence_SET_ITEM(result, 6, PyInt_FromLong(ru.ru_minflt));
-    PyStructSequence_SET_ITEM(result, 7, PyInt_FromLong(ru.ru_majflt));
-    PyStructSequence_SET_ITEM(result, 8, PyInt_FromLong(ru.ru_nswap));
-    PyStructSequence_SET_ITEM(result, 9, PyInt_FromLong(ru.ru_inblock));
-    PyStructSequence_SET_ITEM(result, 10, PyInt_FromLong(ru.ru_oublock));
-    PyStructSequence_SET_ITEM(result, 11, PyInt_FromLong(ru.ru_msgsnd));
-    PyStructSequence_SET_ITEM(result, 12, PyInt_FromLong(ru.ru_msgrcv));
-    PyStructSequence_SET_ITEM(result, 13, PyInt_FromLong(ru.ru_nsignals));
-    PyStructSequence_SET_ITEM(result, 14, PyInt_FromLong(ru.ru_nvcsw));
-    PyStructSequence_SET_ITEM(result, 15, PyInt_FromLong(ru.ru_nivcsw));
+    PyStructSequence_SET_ITEM(result, 2, PyLong_FromLong(ru.ru_maxrss));
+    PyStructSequence_SET_ITEM(result, 3, PyLong_FromLong(ru.ru_ixrss));
+    PyStructSequence_SET_ITEM(result, 4, PyLong_FromLong(ru.ru_idrss));
+    PyStructSequence_SET_ITEM(result, 5, PyLong_FromLong(ru.ru_isrss));
+    PyStructSequence_SET_ITEM(result, 6, PyLong_FromLong(ru.ru_minflt));
+    PyStructSequence_SET_ITEM(result, 7, PyLong_FromLong(ru.ru_majflt));
+    PyStructSequence_SET_ITEM(result, 8, PyLong_FromLong(ru.ru_nswap));
+    PyStructSequence_SET_ITEM(result, 9, PyLong_FromLong(ru.ru_inblock));
+    PyStructSequence_SET_ITEM(result, 10, PyLong_FromLong(ru.ru_oublock));
+    PyStructSequence_SET_ITEM(result, 11, PyLong_FromLong(ru.ru_msgsnd));
+    PyStructSequence_SET_ITEM(result, 12, PyLong_FromLong(ru.ru_msgrcv));
+    PyStructSequence_SET_ITEM(result, 13, PyLong_FromLong(ru.ru_nsignals));
+    PyStructSequence_SET_ITEM(result, 14, PyLong_FromLong(ru.ru_nvcsw));
+    PyStructSequence_SET_ITEM(result, 15, PyLong_FromLong(ru.ru_nivcsw));
 
     if (PyErr_Occurred()) {
         Py_DECREF(result);
@@ -126,7 +123,7 @@ resource_getrlimit(PyObject *self, PyObject *args)
     }
 
     if (getrlimit(resource, &rl) == -1) {
-        PyErr_SetFromErrno(ResourceError);
+        PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
 
@@ -170,20 +167,18 @@ resource_setrlimit(PyObject *self, PyObject *args)
     maxobj = PyTuple_GET_ITEM(limits, 1);
 
 #if !defined(HAVE_LARGEFILE_SUPPORT)
-    rl.rlim_cur = PyInt_AsLong(curobj);
+    rl.rlim_cur = PyLong_AsLong(curobj);
     if (rl.rlim_cur == (rlim_t)-1 && PyErr_Occurred())
         goto error;
-    rl.rlim_max = PyInt_AsLong(maxobj);
+    rl.rlim_max = PyLong_AsLong(maxobj);
     if (rl.rlim_max == (rlim_t)-1 && PyErr_Occurred())
         goto error;
 #else
     /* The limits are probably bigger than a long */
-    rl.rlim_cur = PyLong_Check(curobj) ?
-        PyLong_AsLongLong(curobj) : PyInt_AsLong(curobj);
+    rl.rlim_cur = PyLong_AsLongLong(curobj);
     if (rl.rlim_cur == (rlim_t)-1 && PyErr_Occurred())
         goto error;
-    rl.rlim_max = PyLong_Check(maxobj) ?
-        PyLong_AsLongLong(maxobj) : PyInt_AsLong(maxobj);
+    rl.rlim_max = PyLong_AsLongLong(maxobj);
     if (rl.rlim_max == (rlim_t)-1 && PyErr_Occurred())
         goto error;
 #endif
@@ -198,7 +193,7 @@ resource_setrlimit(PyObject *self, PyObject *args)
             PyErr_SetString(PyExc_ValueError,
                             "not allowed to raise maximum limit");
         else
-            PyErr_SetFromErrno(ResourceError);
+            PyErr_SetFromErrno(PyExc_OSError);
         goto error;
     }
     Py_DECREF(limits);
@@ -242,23 +237,32 @@ resource_methods[] = {
 
 /* Module initialization */
 
+
+static struct PyModuleDef resourcemodule = {
+    PyModuleDef_HEAD_INIT,
+    "resource",
+    NULL,
+    -1,
+    resource_methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
 PyMODINIT_FUNC
-initresource(void)
+PyInit_resource(void)
 {
     PyObject *m, *v;
 
     /* Create the module and add the functions */
-    m = Py_InitModule("resource", resource_methods);
+    m = PyModule_Create(&resourcemodule);
     if (m == NULL)
-        return;
+        return NULL;
 
     /* Add some symbolic constants to the module */
-    if (ResourceError == NULL) {
-        ResourceError = PyErr_NewException("resource.error",
-                                           NULL, NULL);
-    }
-    Py_INCREF(ResourceError);
-    PyModule_AddObject(m, "error", ResourceError);
+    Py_INCREF(PyExc_OSError);
+    PyModule_AddObject(m, "error", PyExc_OSError);
     if (!initialized)
         PyStructSequence_InitType(&StructRUsageType,
                                   &struct_rusage_desc);
@@ -331,16 +335,21 @@ initresource(void)
     PyModule_AddIntConstant(m, "RUSAGE_BOTH", RUSAGE_BOTH);
 #endif
 
+#ifdef RUSAGE_THREAD
+    PyModule_AddIntConstant(m, "RUSAGE_THREAD", RUSAGE_THREAD);
+#endif
+
 #if defined(HAVE_LONG_LONG)
     if (sizeof(RLIM_INFINITY) > sizeof(long)) {
         v = PyLong_FromLongLong((PY_LONG_LONG) RLIM_INFINITY);
     } else
 #endif
     {
-        v = PyInt_FromLong((long) RLIM_INFINITY);
+        v = PyLong_FromLong((long) RLIM_INFINITY);
     }
     if (v) {
         PyModule_AddObject(m, "RLIM_INFINITY", v);
     }
     initialized = 1;
+    return m;
 }
